import 'dart:io';

import 'package:image_picker/image_picker.dart';
import 'package:intl/intl.dart';
import 'package:sghi_core/communities/core/entities/event_types.dart';
import 'package:sghi_core/communities/core/enums.dart';
import 'package:sghi_core/communities/models/message.dart';
import 'package:sghi_core/communities/models/room.dart';
import 'package:sghi_core/communities/models/strings.dart';

///
/// Overrides mime types
///
/// Formatting mimeTypes through lookup
/// didn't work for heic
///
String convertMimeTypes(
  File file,
  String? mimeType,
) {
  if (file.path.toLowerCase().contains('heic')) {
    return 'image/heic';
  }

  if (mimeType == null) {
    throw 'Unsupported Media type for a message';
  }

  return mimeType;
}

Future<File?> selectPhoto({required ImagePicker picker}) async {
  // Has a problem with image path
  // final XFile? imgXFile = await picker.pickImage(
  //   source: ImageSource.gallery, // We will do gallery for now
  // );

  // ignore: deprecated_member_use
  final PickedFile? image = await picker.getImage(source: ImageSource.gallery);

  if (image == null) return null;

  final File file = File(image.path);

  /// Consider compressing the image
  return file;
}

List<int> fibonacci(int n) {
  if (n == 0) {
    return <int>[0];
  }
  if (n == 1) {
    return <int>[0, 1];
  }

  final List<int> series = fibonacci(n - 1);

  series.add(series[series.length - 1] + series[series.length - 2]);

  return series;
}

/// Processes the initial room data after the sync API is called the first time
///
/// Used for processing totally new events from the server
///
/// Adds things like the group name and invite status
Map<String, Room>? enrichRooms(
  Map<String, Room>? roomsData, {
  bool invite = false,
}) {
  if (roomsData?.isNotEmpty ?? false) {
    final Map<String, Room> parsedRooms = <String, Room>{
      for (final String id in roomsData!.keys)
        if (roomsData[id] != null)
          id: enrichRoom(room: roomsData[id]!, roomID: id, invite: invite),
    };

    return parsedRooms;
  }
  return null;
}

/// Enriches a single room object with the group name, avatar URL, and invite status
Room enrichRoom({
  required Room room,
  required String roomID,
  required bool invite,
}) {
  final List<Message?>? stateEvents =
      invite ? room.inviteState?.events : room.state?.events;

  Message? nameEvent;

  final Message? stateNameEvent = stateEvents?.firstWhere(
    (Message? m) => m?.eventType == EventTypes.name,
    orElse: () => Message.initial(),
  );

  final Message? avatar = stateEvents?.firstWhere(
    (Message? m) => m?.eventType == EventTypes.avatar,
    orElse: () => Message.initial(),
  );

  if (stateNameEvent == Message.initial()) {
    // Attempt to find the group name in the state events
    final Message? timelineNameEvent = room.timeline?.events?.firstWhere(
      (Message? m) => m?.eventType == EventTypes.name,
      orElse: () => Message.initial(),
    );

    nameEvent = timelineNameEvent;
  } else {
    nameEvent = stateNameEvent;
  }

  final Room newRoom = room.copyWith(
    name: nameEvent?.content?.name,
    roomID: roomID,
    invite: invite,
    avatarUri: avatar?.content?.url ?? UNKNOWN,
  );

  return newRoom;
}

/// Loops through the new events from Matrix and runs updates on the old events
///
/// Used for subsequent room updates after the initial sync
Map<String, Room>? updateRoomData({
  // From server
  required Map<String, Room>? fromServer,

  // From state
  required Map<String, Room>? fromState,

  // Whether the room is an invite or not
  required bool invite,
}) {
  if (fromServer != null) {
    /// Loop through all rooms from the server
    final Map<String, Room> updatedRooms = <String, Room>{};

    fromServer.forEach((String rID, Room r) {
      if (fromState?.containsKey(rID) ?? false) {
        updatedRooms[rID] = processRoomUpdates(
          serverRoom: r,
          stateRoom: fromState![rID]!,
        );
      } else {
        updatedRooms[rID] = enrichRoom(room: r, roomID: rID, invite: invite);
      }
    });

    // Update state with new/updated rooms from server
    final Map<String, Room> newState =
        Map<String, Room>.from(fromState ?? <String, Room>{})
          ..addAll(updatedRooms);

    return newState.isNotEmpty ? newState : null;
  }

  return fromState;
}

/// Cleans up room updates by removing duplicate events
Room processRoomUpdates({required Room serverRoom, required Room stateRoom}) {
  // Extract state and timeline events, and combine them
  final List<Message?> allStateEvents = <Message?>[
    ...?serverRoom.state?.events,
    ...?stateRoom.state?.events
  ];

  // Extract state and timeline events and combine them
  final List<Message?> allTimelineEvents = <Message?>[
    ...?serverRoom.timeline?.events,
    ...?stateRoom.timeline?.events
  ];

  // Extract state and timeline event IDs. Used to remove duplicate events
  final Set<String?> stateEventIDs =
      allStateEvents.map((Message? e) => e?.eventID).toSet();
  final Set<String?> timelineEventIDs =
      allTimelineEvents.map((Message? e) => e?.eventID).toSet();

  // Remove duplicate state events
  allStateEvents.retainWhere((Message? m) => stateEventIDs.remove(m?.eventID));

  // Remove duplicate timeline events
  allTimelineEvents
      .retainWhere((Message? m) => timelineEventIDs.remove(m?.eventID));

  final Room processedRoom = stateRoom.copyWith.call(
    timeline: stateRoom.timeline?.copyWith.call(events: allTimelineEvents),
    state: stateRoom.state?.copyWith.call(events: allStateEvents),
    name: (serverRoom.name?.isNotEmpty ?? false)
        ? serverRoom.name
        : stateRoom.name,
    topic: (serverRoom.topic?.isNotEmpty ?? false)
        ? serverRoom.topic
        : stateRoom.topic,
  );

  return processedRoom;
}

/// Retrieves a name's initials
///
/// Example 'Abiud Orina' returns 'AO'
String getInitials(String string) {
  if (string.isEmpty) return '';
  final String initials =
      string.trim().split(RegExp(' +')).map((String s) => s[0]).join();

  return (initials.length > 2) ? initials.substring(0, 2) : initials;
}

/// Truncates the unnecessary parts of a username
///
/// For example @abiudrn:matrix.org returns @abiudrn
String formatUsername(String? name) =>
    (name?.split(':').first ?? 'No name').split(':').first;

/// Formats a timestamp
String formatTimestamp({
  required int? timestamp,
  bool showTime = true,
  bool showDate = true,
}) {
  if (timestamp != null) {
    /// Format timestamp
    final DateTime formattedTimestamp =
        DateTime.fromMillisecondsSinceEpoch(timestamp);
    final String formattedDate =
        DateFormat('d MMM, yy').format(formattedTimestamp);
    final String formattedTime = DateFormat('jm').format(formattedTimestamp);

    return '${showDate ? formattedDate : ''} ${showTime ? 'at $formattedTime' : ''}';
  }

  return '';
}

/// Returns a human readable String for membership events
String processMembership(
  MembershipEventTypes? type,
  String name,
  String sender,
) {
  final String? membershipAction = type?.name;

  if ((membershipAction?.isNotEmpty ?? false) &&
      type != MembershipEventTypes.unknown) {
    switch (type) {
      case MembershipEventTypes.invite:
        return '$sender $membershipAction $name';
      case MembershipEventTypes.join:
        return '$name $membershipAction';
      case MembershipEventTypes.leave:
        return '$name $membershipAction';
      case MembershipEventTypes.ban:
        return '$name was $membershipAction';
      case MembershipEventTypes.knock:
        return '$name $membershipAction';
      default:
        return '';
    }
  }
  return '';
}

// Sort messages events in the order that they happened
List<Message?>? organizeEventTimeline(List<Message?>? events) {
  final List<Message?>? sortedEvents = events;
  if (sortedEvents != null && sortedEvents.isEmpty) return <Message>[];

  sortedEvents?.sort(
    (Message? a, Message? b) => b?.timeStamp?.compareTo(a?.timeStamp ?? 0) ?? 0,
  );

  return sortedEvents;
}

/// Returns a human readable format of a Matrix timestamp
String formatMessageTimestamp(int? timestamp) {
  if (timestamp == null) {
    return '';
  }

  final DateTime now = DateTime.now();
  final DateTime messageTime = DateTime.fromMillisecondsSinceEpoch(timestamp);

  final Duration difference = now.difference(messageTime);

  if (difference.inSeconds < 5) {
    return 'Just now';
  } else if (difference.inSeconds < 60) {
    return 'A few seconds ago';
  } else if (difference.inMinutes < 60) {
    return '${difference.inMinutes} ${difference.inMinutes == 1 ? 'minute' : 'minutes'} ago';
  } else if (difference.inHours < 24 && now.day == messageTime.day) {
    final DateFormat formatter = DateFormat('h:mm a');
    return 'Today at ${formatter.format(messageTime)}';
  } else if (difference.inHours < 24 && now.day - messageTime.day == 1) {
    final DateFormat formatter = DateFormat('h:mm a');
    return 'Yesterday at ${formatter.format(messageTime)}';
  } else if (difference.inDays < 7) {
    final DateFormat formatter = DateFormat('EEEE');
    return formatter.format(messageTime);
  } else if (difference.inDays < 365) {
    final DateFormat formatter = DateFormat('d MMMM');
    return formatter.format(messageTime);
  } else {
    final DateFormat formatter = DateFormat('d MMM yyyy');
    return formatter.format(messageTime);
  }
}
