import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/communities/core/enums.dart';

part 'message.freezed.dart';
part 'message.g.dart';

/// Represents a Matrix message.
///
/// The [eventType] field represents the type of the event.
/// The [sender] field represents the unique identifier of the sender.
/// The [content] field represents the content of the message.
/// The [timeStamp] field represents the timestamp of the message.
/// The [eventID] field represents the unique identifier of the event.
/// The [roomID] field represents the unique identifier of the room.
/// The [stateKey] field represents the state key of the message.
///
/// This class corresponds to the message object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#message-event.
///
/// To create an instance of [Message], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [Message] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final message = Message.fromJson(jsonData);
/// final eventType = message.eventType;
/// final sender = message.sender;
/// final content = message.content;
/// ```
@freezed
class Message with _$Message {
  /// Creates a [Message] instance.
  ///
  /// The [eventType] field represents the type of the event.
  /// The [sender] field represents the unique identifier of the sender.
  /// The [content] field represents the content of the message.
  /// The [timeStamp] field represents the timestamp of the message.
  /// The [eventID] field represents the unique identifier of the event.
  /// The [roomID] field represents the unique identifier of the room.
  /// The [stateKey] field represents the state key of the message.
  factory Message({
    @JsonKey(name: 'type') String? eventType,
    @JsonKey(name: 'sender') String? sender,
    @JsonKey(name: 'content') MessageContent? content,
    @JsonKey(name: 'origin_server_ts') int? timeStamp,
    @JsonKey(name: 'event_id') String? eventID,
    @JsonKey(name: 'room_id') String? roomID,
    @JsonKey(name: 'state_key') String? stateKey,
  }) = _Message;

  /// Creates a [Message] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing the message.
  factory Message.fromJson(Map<String, dynamic> json) =>
      _$MessageFromJson(json);

  /// Creates an initial [Message] instance.
  ///
  /// All fields are initialized with default values.
  factory Message.initial() => Message(
        eventType: UNKNOWN,
        sender: UNKNOWN,
        content: MessageContent.initial(),
        timeStamp: 0,
        eventID: UNKNOWN,
        stateKey: UNKNOWN,
        roomID: UNKNOWN,
      );
}

/// Represents the content of a message in Matrix.
///
/// The [msgType] field represents the type of the message.
/// The [body] field represents the body of the message.
/// The [topic] field represents the topic of the message.
/// The [name] field represents the name of the message.
/// The [guestAccess] field represents the guest access of the message.
/// The [historyVisibility] field represents the history visibility of the message.
/// The [joinRule] field represents the join rule of the message.
/// The [alias] field represents the alias of the message.
/// The [creator] field represents the creator of the message.
/// The [roomVersion] field represents the room version of the message.
/// The [users] field represents the users associated with the message.
/// The [membership] field represents the membership event type of the message.
/// The [displayName] field represents the display name of the message.
/// The [url] field represents the URL of the message.
///
/// This class corresponds to the message content object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#message-content.
///
/// To create an instance of [MessageContent], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [MessageContent] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final content = MessageContent.fromJson(jsonData);
/// final msgType = content.msgType;
/// final body = content.body;
/// final topic = content.topic;
/// ```
@freezed
class MessageContent with _$MessageContent {
  /// Creates a [MessageContent] instance.
  ///
  /// The [msgType] field represents the type of the message.
  /// The [body] field represents the body of the message.
  /// The [topic] field represents the topic of the message.
  /// The [name] field represents the name of the message.
  /// The [guestAccess] field represents the guest access of the message.
  /// The [historyVisibility] field represents the history visibility of the message.
  /// The [joinRule] field represents the join rule of the message.
  /// The [alias] field represents the alias of the message.
  /// The [creator] field represents the creator of the message.
  /// The [roomVersion] field represents the room version of the message.
  /// The [users] field represents the users associated with the message.
  /// The [membership] field represents the membership event type of the message.
  /// The [displayName] field represents the display name of the message.
  /// The [url] field represents the URL of the message.
  factory MessageContent({
    @JsonKey(name: 'msgtype') String? msgType,
    @JsonKey(name: 'body') String? body,
    @JsonKey(name: 'topic') String? topic,
    @JsonKey(name: 'name') String? name,
    @JsonKey(name: 'guest_access') String? guestAccess,
    @JsonKey(name: 'history_visibility') String? historyVisibility,
    @JsonKey(name: 'join_rule') String? joinRule,
    @JsonKey(name: 'alias') String? alias,
    @JsonKey(name: 'creator') String? creator,
    @JsonKey(name: 'room_version') String? roomVersion,
    @JsonKey(name: 'users') Map<String, dynamic>? users,
    @JsonKey(name: 'membership') MembershipEventTypes? membership,
    @JsonKey(name: 'displayname') String? displayName,

    // For Images
    @JsonKey(name: 'url') String? url,
  }) = _MessageContent;

  /// Creates a [MessageContent] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing the message content.
  factory MessageContent.fromJson(Map<String, dynamic> json) =>
      _$MessageContentFromJson(json);

  /// Creates an initial [MessageContent] instance.
  ///
  /// All fields are initialized with default values.
  factory MessageContent.initial() => MessageContent(
        msgType: UNKNOWN,
        body: UNKNOWN,
        topic: UNKNOWN,
        name: UNKNOWN,
        guestAccess: UNKNOWN,
        historyVisibility: UNKNOWN,
        joinRule: UNKNOWN,
        alias: UNKNOWN,
        creator: UNKNOWN,
        roomVersion: UNKNOWN,
        users: <String, dynamic>{},
        membership: MembershipEventTypes.unknown,
        displayName: UNKNOWN,
        url: UNKNOWN,
      );
}
