import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';

part 'user.freezed.dart';
part 'user.g.dart';

/// Represents a Matrix user
///
/// The [userID] field contains the unique identifier of the user.
///
/// The [deviceId] field contains the identifier of the user's device.
///
/// The [displayName] field contains the display name of the user.
///
/// The [accessToken] field contains the access token associated with the user.
///
/// The [avatarUri] field contains the URI to the user's avatar.
///
/// The [isSignedIn] field indicates whether the user is currently signed in.
///
/// This class corresponds to the user object defined in the Matrix API specification.
/// For more details, refer to the API documentation at https://matrix.org/docs/spec/client_server/latest#user-object.
///
/// To create an instance of [User], use the provided factory constructors or the `fromJson` method.
/// It is also possible to create an initial instance of [User] using the `initial` factory constructor.
///
/// Example usage:
/// ```dart
/// final user = User.fromJson(jsonData);
/// final userID = user.userID;
/// final displayName = user.displayName;
/// ```
@freezed
class User with _$User {
  /// Creates a [User] instance.
  ///
  /// The [userID] field represents the unique identifier of the user.
  ///
  /// The [deviceId] field represents the identifier of the user's device.
  ///
  /// The [displayName] field represents the display name of the user.
  ///
  /// The [accessToken] field represents the access token associated with the user.
  ///
  /// The [avatarUri] field represents the URI to the user's avatar.
  ///
  /// The [isSignedIn] field indicates whether the user is currently signed in.
  factory User({
    @JsonKey(name: 'user_id') String? userID,
    @JsonKey(name: 'device_id') String? deviceId,
    @JsonKey(name: 'display_name') String? displayName,
    @JsonKey(name: 'access_token') String? accessToken,
    @JsonKey(name: 'avatar_url') String? avatarUri,
    @JsonKey(name: 'isSignedIn') bool? isSignedIn,
  }) = _User;

  /// Creates a [User] instance from a JSON map.
  ///
  /// The [json] parameter should be a valid JSON map representing user information.
  factory User.fromJson(Map<String, dynamic> json) => _$UserFromJson(json);

  /// Creates an initial [User] instance.
  ///
  /// The [userID], [deviceId], [accessToken], [displayName], and [avatarUri] fields are initialized with [UNKNOWN].
  ///
  /// The [isSignedIn] field is initialized as `false`.
  factory User.initial() => User(
        userID: UNKNOWN,
        deviceId: UNKNOWN,
        accessToken: UNKNOWN,
        displayName: UNKNOWN,
        avatarUri: UNKNOWN,
        isSignedIn: false,
      );
}
