import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';
import 'package:http/http.dart' as http;
import 'package:image_picker/image_picker.dart';
import 'package:mockito/mockito.dart';
import 'package:sghi_core/communities/core/endpoints.dart';
import 'package:sghi_core/flutter_graphql_client/flutter_graphql_client.dart';
import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

import 'response_mocks.dart';

/// a short client for providing custom responses
///
/// a good use case is when you want to return error responses
class MockShortGraphQlClient extends IGraphQlClient {
  MockShortGraphQlClient();
  MockShortGraphQlClient.withResponse(
    String idToken,
    String endpoint,
    this.response,
  ) {
    super.idToken = idToken;
    super.endpoint = endpoint;
  }

  late final Response response;

  @override
  Future<Response> callRESTAPI({
    required String endpoint,
    required String method,
    Map<String, dynamic>? variables,
    Duration? timeout,
    Map<String, String>? customHeaders,
  }) {
    return Future<Response>.value(response);
  }

  @override
  Future<Response> query(String queryString, Map<String, dynamic> variables,
      {Duration? timeout}) {
    return Future<Response>.value(response);
  }
}

class MockGraphQlClient extends Mock implements GraphQlClient {
  @override
  Future<http.Response> callRESTAPI({
    required String endpoint,
    required String method,
    Map<String, dynamic>? variables,
    Map<String, String>? customHeaders,
    Duration? timeout,
  }) async {
    if (endpoint.contains(loginEndpoint)) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(loginResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains(syncEndpoint)) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(syncResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains('joined_members')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(joinedMembersResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains('/send/m.room.message/')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'event_id': '16741180035XJnPW:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('m.room.power_levels')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(powerLevelsResponseMock),
          201,
        ),
      );
    }

    if (endpoint.contains('/invite')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{}),
          201,
        ),
      );
    }

    if (endpoint.contains('/join')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'room_id': '!NvYSqaASzlfRpFCMtr:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('/leave')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{}),
          201,
        ),
      );
    }

    if (endpoint.contains('/kick')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{}),
          201,
        ),
      );
    }

    if (endpoint.contains('/redact')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'event_id': '16741180035XJnPW:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('/createRoom')) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'room_id': '!NvYSqaASzlfRpFCMtr:chat.savannahghi.org',
          }),
          201,
        ),
      );
    }

    if (endpoint.contains('/ban')) {
      return Future<http.Response>.value(
        http.Response(json.encode(<String, dynamic>{}), 201),
      );
    }

    if (endpoint.contains('/unban')) {
      return Future<http.Response>.value(
        http.Response(json.encode(<String, dynamic>{}), 201),
      );
    }

    if (endpoint.contains('/report/')) {
      return Future<http.Response>.value(
        http.Response(json.encode(<String, dynamic>{}), 201),
      );
    }

    if (endpoint.contains('/media/v3/download/')) {
      final String dir = Directory.current.path;
      final String imgPath = '$dir/test/test_resources/test_file.png';
      final ByteData imgData = await rootBundle.load(imgPath);
      final Uint8List uint8list = imgData.buffer.asUint8List();

      return Future<http.Response>.value(
        http.Response(json.encode(uint8list), 201),
      );
    }

    if (endpoint.contains(searchMembersEndpoint)) {
      return Future<http.Response>.value(
        http.Response(
          json.encode(<String, dynamic>{
            'results': <Map<String, dynamic>>[loginResponseMock],
          }),
          201,
        ),
      );
    }

    return Future<http.Response>.value(
      http.Response(
        json.encode(<String, dynamic>{'data': <String, dynamic>{}}),
        200,
      ),
    );
  }

  @override
  Map<String, dynamic> toMap(http.Response? response) {
    final dynamic res = json.decode(response!.body);

    if (res is Map) return res as Map<String, dynamic>;
    // ignore: avoid_dynamic_calls
    return res[0] as Map<String, dynamic>;
  }
}

class ImagePickerMock extends Mock implements ImagePicker {
  @override
  Future<PickedFile?> getImage(
      {required ImageSource source,
      double? maxWidth,
      double? maxHeight,
      int? imageQuality,
      CameraDevice preferredCameraDevice = CameraDevice.rear}) {
    final String dir = Directory.current.path;
    final String imgPath = '$dir/test/test_resources/test_file.png';

    final PickedFile selectedImage = PickedFile(imgPath);

    return Future<PickedFile>.value(selectedImage);
  }
}
